@@ -134,6 +134,56 @@ _topLevelMDStreename = {
 my_ip_address = [get_ip()]
 
 
+# https://stackoverflow.com/questions/492519/timeout-on-a-function-call
+def runFunctionCatchExceptions(func, *args, **kwargs):
+    try:
+        result = func(*args, **kwargs)
+    except Exception as  message:
+        return "exception", message
+
+    return "RESULT", result
+
+
+def runFunctionWithTimeout(func, *args, **kwargs):
+    timeout = 10 #s
+    import threading, time
+    class InterruptableThread(threading.Thread):
+        def __init__(self):
+            threading.Thread.__init__(self)
+            self.result = None
+        def run(self):
+            self.result = runFunctionCatchExceptions(func, *args, **kwargs)
+    it = InterruptableThread()
+    it.start()
+    it.join(timeout)
+ 
+    if it.is_alive():
+        return False, None
+
+    if it.result[0] == "exception":
+        raise it.result[1]
+
+    return True, it.result[1]
+    
+#raise error of the tree cannot be opened in 5 seconds
+def safeOpenTree(conn, tree, shot):
+    if conn is None:
+        raise OMFITexception('MDS+ connection was unsuccesful')
+        
+    finished, _ = runFunctionWithTimeout(conn.openTree,  tree, shot)
+    if not finished:
+        raise OMFITexception('Opening of MDS+ tree was unsuccesful')
+        
+#raise error of the tree cannot be connected in 5 seconds
+def safeMDSConnect(server):
+    finished, conn = runFunctionWithTimeout(MDSplus.Connection, server)
+
+    if not finished or conn is None:
+        raise OMFITexception('MDS+ connection was unsuccesful')
+
+    return conn
+
+    
 def ip_check_reset():
     """reset connections when IP changes, this is necessary to avoid long timeout hardcoded in C part of MDS+"""
     current_ip = get_ip()
@@ -268,10 +318,10 @@ def MDSplusConnection(server, cached=True, quiet=True):
         tmp = OMFITmdsConnectionBase(quiet=quiet)
         mds_connection_cache_ID = ('connection', tunneled_server, None, None)
         if mds_connection_cache_ID not in tmp:
-            tmp[mds_connection_cache_ID] = MDSplus.Connection(tunneled_server)
+            tmp[mds_connection_cache_ID] = safeMDSConnect(tunneled_server)
         return tmp[mds_connection_cache_ID]
     else:
-        return MDSplus.Connection(tunneled_server)
+        return safeMDSConnect(tunneled_server)
 
 
 class OMFITmdsObjects(object):
@@ -508,7 +558,8 @@ class OMFITmdsConnectionBaseClass(SortedDict):
                 'not be respected by MDSplus version {}'.format(timeout, getattr(MDSplus, '__version__', '0.0.0'))
             )
         self.timeout = timeout
-
+        self.last_open_tree = {}
+        
     def mdsvalue(self, server, treename, shot, what, timeout=None):
         warnings.filterwarnings('ignore', message=r"tostring\(\) is deprecated\. Use tobytes\(\) instead\.", category=DeprecationWarning)
 
@@ -545,12 +596,13 @@ class OMFITmdsConnectionBaseClass(SortedDict):
                         printi('* Re-connecting to: ' + server)
                         time.sleep(np.random.rand())
                     printd('* Connecting to: ' + server, topic='MDS')
-                    self[mds_connection_cache_ID] = MDSplus.Connection(server)
-
+                    self[mds_connection_cache_ID] = safeMDSConnect(server)
+                    self.last_open_tree = {}
                 if treename is None and shot is not None:
+                    self.last_open_tree = {}
                     try:
                         signal = b2s(self[mds_connection_cache_ID].get('findsig("' + signal + '",_fstree)', timeout=timeout).value)
-                        treename = self[mds_connection_cache_ID].get('_fstree', timeout=timeout).value
+                        treename = b2s(self[mds_connection_cache_ID].get('_fstree', timeout=timeout).value)
                     except Exception:
                         pass
 
@@ -591,7 +643,11 @@ class OMFITmdsConnectionBaseClass(SortedDict):
                             )
 
                         elif treename is not None:
-                            self[mds_connection_cache_ID].openTree(treename, int(shot))
+                            #Open new tree only if it not already opened
+                            if self.last_open_tree.get(mds_connection_cache_ID,'') != (treename,int(shot)) :
+                                safeOpenTree(self[mds_connection_cache_ID], treename, int(shot))
+                                self.last_open_tree[mds_connection_cache_ID] = treename, int(shot)
+                                
                             try:
                                 queryResult = mdsplus_data(self[mds_connection_cache_ID].get(pre + signal + post, timeout=timeout))
                                 printd(f'* Retrieving data {treename} for shot {shot}', topic='MDS')
@@ -737,8 +793,8 @@ class OMFITmdsConnectionBaseClass(SortedDict):
         """
         mds_connection_cache_ID = mds_connection_cache_ID_generator('write', server, treename, shot)
         if mds_connection_cache_ID not in self:
-            self[mds_connection_cache_ID] = MDSplus.Connection(server)
-            self[mds_connection_cache_ID].openTree(treename, shot)
+            self[mds_connection_cache_ID] = safeMDSConnect(server)
+            safeOpenTree(self[mds_connection_cache_ID],treename, shot)
         conn = self[mds_connection_cache_ID]
 
         # generate TDI expression for writing data to MDS+
@@ -794,14 +850,14 @@ class OMFITmdsConnectionBaseClass(SortedDict):
 
         mds_connection_cache_ID = mds_connection_cache_ID_generator('write', server, treename, shot)
         if mds_connection_cache_ID not in self:
-            self[mds_connection_cache_ID] = MDSplus.Connection(server)
+            self[mds_connection_cache_ID] = safeMDSConnect(server)
             # create pulse from model tree
             self[mds_connection_cache_ID].get(f"tcl('set tree {treename}')")
             self[mds_connection_cache_ID].get(f"tcl('create pulse {shot}')")
             self[mds_connection_cache_ID].get(f"tcl('write')")
             self[mds_connection_cache_ID].get(f"tcl('close')")
             # open tree
-            self[mds_connection_cache_ID].openTree(treename, shot)
+            safeOpenTree(self[mds_connection_cache_ID], treename, shot)
             if subtree:
                 self[mds_connection_cache_ID].setDefault(subtree)
         conn = self[mds_connection_cache_ID]
@@ -919,7 +975,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
                     if fallback:
                         printi('* Re-connecting to: ' + server)
                     printd('* Connecting to: ' + server, topic='MDS')
-                    self[mds_connection_cache_ID] = MDSplus.Connection(server)
+                    self[mds_connection_cache_ID] = safeMDSConnect(server)
 
                 return self[mds_connection_cache_ID].get("tcl(" + repr(tcl) + ")")
 
@@ -931,7 +987,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
         username, password, server, port = parse_server(test_server)
         if test_connection(username, server, port):
             try:
-                MDSplus.Connection(test_server)
+                safeMDSConnect(test_server)
                 return True
             except Exception as _excp:
                 if 'Error connecting to' in str(_excp):
