@@ -51,14 +51,27 @@ class OMFITosborneProfile(OMFITmds):
     [`tree`]['PROFDB_PED']['P<time>_<runid>']
     """
 
-    def __init__(self, server='DIII-D', treename='PEDESTAL', shot=None, time=None, runid=None):
+    def __init__(self, server='DIII-D', treename='auto', shot=None, time=None, runid=None):
+        if treename == 'auto':
+            if is_device(server, 'DIII-D'):
+                treename = 'PEDESTAL'
+            elif is_device(server, ('NSTX', 'NSTX-U')):
+                treename = 'PROFDB_PED'
         self.OMFITproperties = {}
         OMFITmds.__init__(self, server, treename, shot)
+        self.OMFITproperties['treename'] = treename
         self.OMFITproperties['time'] = time
         self.OMFITproperties['runid'] = runid
         self.OMFITproperties['shot'] = shot
 
     @property
+    def treename(self):
+        return self.OMFITproperties['treename']
+
+    @treename.setter
+    def treename(self, value):
+        self.OMFITproperties['treename'] = value
+    @property
     def time(self):
         return self.OMFITproperties['time']
 
@@ -97,9 +110,12 @@ class OMFITosborneProfile(OMFITmds):
     @dynaLoad
     def load(self):
         OMFITmds.load(self)
-        if 'P%s_%s' % (self.time, self.runid.upper()) not in self['PROFDB_PED']:
+        profdb_ped = self
+        if self.treename == 'PEDESTAL':
+            profdb_ped = self['PROFDB_PED']
+        if 'P%s_%s' % (self.time, self.runid.upper()) not in profdb_ped:
             raise RuntimeError('Time: %s, Runid: %s does not exist for shot %s' % (self.time, self.runid, self.shot))
-        tmp = self['PROFDB_PED']['P%s_%s' % (self.time, self.runid.upper())]
+        tmp = profdb_ped['P%s_%s' % (self.time, self.runid.upper())]
         self.clear()
         self.update(tmp)
 
