@@ -893,6 +893,41 @@ If this file is not available to you contact the GA computer group: iris-support
 
         return connection
 
+    def get_databases(self):
+        '''
+        Return a list of databases on the server
+        '''
+        return sorted(list(self.select('select name from sys.databases', by_column=True)['name']))
+
+    def get_tables(self):
+        '''
+        Return a list of tables on the given database
+        '''
+        return sorted(list(self.select('select table_name from information_schema.tables', by_column=True)['table_name']))
+
+    def get_columns(self, table):
+        '''
+        Return a list of columns in the given table
+        '''
+        if table not in self.get_tables():
+            raise RuntimeError(f'Table {table} not in {self.get_tables()}')
+        return sorted(
+            list(
+                self.select(f"select column_name from information_schema.columns where table_name='{table}'", by_column=True)['column_name']
+            )
+        )
+
+    def get_db_structure(self):
+        '''
+        Return a nested list of tables and columns of those tables within the current database
+        '''
+        result = {}
+        for table in self.get_tables():
+            result[table] = []
+            for column in self.get_columns(table):
+                result[table].append(column)
+        return result
+
 
 def available_efits_from_rdb(scratch, device, shot, default_snap_list=None, format='{tree}', **kw):
     """
