@@ -135,6 +135,8 @@ class OMFITobject(object):
 
     :param \**kw: dictionary intended to be used by subclassing omfit_classes. Ignored by OMFITobject.
     """
+    # static variable when the objects were cleaned the last time
+    cleanup_time = [0]
 
     def __init__(
         self,
@@ -153,7 +155,10 @@ class OMFITobject(object):
 
         # cleanup other OMFITobject files that are not referenced anymore
         if not len(OMFITaux['prun_process']):
-            _cleanup()
+            # do not call cleaning more often then once per second
+            if time.time() - self.cleanup_time[0] > 1:
+                _cleanup()
+                self.cleanup_time[0] = time.time()
 
         # store extra arguments that the user passed, so that these can be used to generate the OMFITsave.txt file
         self.OMFITproperties = {}
